@@ -19,7 +19,20 @@
    presents a dialog to the user and lets them copy those block
    files into the project, making it self-contained.
 
-**********************************************************************/
+********************************************************************//**
+
+\class AliasedFile
+\brief An audio file that is referenced (pointed into) directly from
+an Audacity .aup file rather thna Audacity having its own copies of the
+data.
+
+*//*****************************************************************//**
+
+\class DependencyDialog
+\brief DependencyDialog shows dependencies of an AudacityProject on 
+AliasedFile s.
+
+*//********************************************************************/
 
 #include "Audacity.h"
 #include "Dependencies.h"
@@ -81,16 +94,12 @@ static void GetAllSeqBlocks(AudacityProject *project,
 // tracks and replace each aliased block file with its replacement.
 // Note that this code respects reference-counting and thus the
 // process of making a project self-contained is actually undoable.
-static void ReplaceBlockFiles(AudacityProject *project,
+static void ReplaceBlockFiles(BlockPtrArray &blocks,
                               ReplacedBlockFileHash &hash)
+// NOFAIL-GUARANTEE
 {
-   //const auto &dirManager = project->GetDirManager();
-   BlockPtrArray blocks;
-   GetAllSeqBlocks(project, &blocks);
-
-   int i;
-   for (i = 0; i < (int)blocks.size(); i++) {
-      auto &f = blocks[i]->f;
+   for (const auto &pBlock : blocks) {
+      auto &f = pBlock->f;
       const auto src = &*f;
       if (hash.count( src ) > 0) {
          const auto &dst = hash[src];
@@ -156,13 +165,14 @@ void FindDependencies(AudacityProject *project,
 // all of those alias block files with disk block files.
 static void RemoveDependencies(AudacityProject *project,
                                AliasedFileArray &aliasedFiles)
+// STRONG-GUARANTEE
 {
    const auto &dirManager = project->GetDirManager();
 
    ProgressDialog progress
       (_("Removing Dependencies"),
       _("Copying audio data into project..."));
-   int updateResult = eProgressSuccess;
+   auto updateResult = ProgressResult::Success;
 
    // Hash aliasedFiles based on their full paths and
    // count total number of bytes to process.
@@ -198,6 +208,8 @@ static void RemoveDependencies(AudacityProject *project,
          BlockFilePtr newBlockFile;
          {
             SampleBuffer buffer(len, format);
+            // We tolerate exceptions from NewSimpleBlockFile and so we
+            // can allow exceptions from ReadData too
             f->ReadData(buffer.ptr(), format, 0, len);
             newBlockFile =
                dirManager->NewSimpleBlockFile(buffer.ptr(), len, format);
@@ -209,16 +221,19 @@ static void RemoveDependencies(AudacityProject *project,
          // Update the progress bar
          completedBytes += SAMPLE_SIZE(format) * len;
          updateResult = progress.Update(completedBytes, totalBytesToProcess);
-         if (updateResult != eProgressSuccess)
-           break;
+         if (updateResult != ProgressResult::Success)
+            // leave the project unchanged
+            return;
       }
    }
 
+   // COMMIT OPERATIONS needing NOFAIL-GUARANTEE:
+
    // Above, we created a SimpleBlockFile contained in our project
    // to go with each AliasBlockFile that we wanted to migrate.
    // However, that didn't actually change any references to these
    // blockfiles in the Sequences, so we do that next...
-   ReplaceBlockFiles(project, blockFileHash);
+   ReplaceBlockFiles(blocks, blockFileHash);
 }
 
 //
@@ -276,7 +291,7 @@ BEGIN_EVENT_TABLE(DependencyDialog, wxDialogWrapper)
    EVT_LIST_ITEM_DESELECTED(FileListID, DependencyDialog::OnList)
    EVT_BUTTON(CopySelectedFilesButtonID, DependencyDialog::OnCopySelectedFiles)
    EVT_SIZE(DependencyDialog::OnSize)
-   EVT_BUTTON(wxID_NO, DependencyDialog::OnNo) // mIsSaving ? "Cancel Save" : "Save without Copying"
+   EVT_BUTTON(wxID_NO, DependencyDialog::OnNo) // mIsSaving ? "Cancel Save" : "Save Without Copying"
    EVT_BUTTON(wxID_YES, DependencyDialog::OnYes) // "Copy All Files (Safer)"
    EVT_BUTTON(wxID_CANCEL, DependencyDialog::OnCancel)  // "Cancel Save"
 END_EVENT_TABLE()
@@ -308,20 +323,26 @@ DependencyDialog::DependencyDialog(wxWindow *parent,
    PopulateOrExchange(S);
 }
 
-const wxString kStdMsg =
+static const wxString kStdMsg()
+{
+   return
 _("Copying these files into your project will remove this dependency.\
 \nThis is safer, but needs more disk space.");
+}
 
-const wxString kExtraMsgForMissingFiles =
+static const wxString kExtraMsgForMissingFiles()
+{
+   return
 _("\n\nFiles shown as MISSING have been moved or deleted and cannot be copied.\
 \nRestore them to their original location to be able to copy into project.");
+}
 
 void DependencyDialog::PopulateOrExchange(ShuttleGui& S)
 {
    S.SetBorder(5);
    S.StartVerticalLay();
    {
-      mMessageStaticText = S.AddVariableText(kStdMsg, false);
+      mMessageStaticText = S.AddVariableText(kStdMsg(), false);
 
       S.StartStatic(_("Project Dependencies"));
       {
@@ -347,7 +368,7 @@ void DependencyDialog::PopulateOrExchange(ShuttleGui& S)
       {
          if (mIsSaving) {
             S.Id(wxID_CANCEL).AddButton(_("Cancel Save"));
-            S.Id(wxID_NO).AddButton(_("Save without Copying"));
+            S.Id(wxID_NO).AddButton(_("Save Without Copying"));
          }
          else
             S.Id(wxID_NO).AddButton(_("Do Not Copy"));
@@ -420,9 +441,9 @@ void DependencyDialog::PopulateList()
       ++i;
    }
 
-   wxString msg = kStdMsg;
+   wxString msg = kStdMsg();
    if (mHasMissingFiles)
-      msg += kExtraMsgForMissingFiles;
+      msg += kExtraMsgForMissingFiles();
    mMessageStaticText->SetLabel(msg);
 
    if (mCopyAllFilesButton)
@@ -473,22 +494,22 @@ void DependencyDialog::OnYes(wxCommandEvent & WXUNUSED(event))
 
 void DependencyDialog::OnCopySelectedFiles(wxCommandEvent & WXUNUSED(event))
 {
-   AliasedFileArray aliasedFilesToDelete;
+   AliasedFileArray aliasedFilesToDelete, remainingAliasedFiles;
 
    long i = 0;
-   for(auto iter = mAliasedFiles.begin(); iter != mAliasedFiles.end();) {
-      if (mFileListCtrl->GetItemState(i, wxLIST_STATE_SELECTED)) {
-         // Two-step move could be simplified when all compilers have C++11 vector
-         aliasedFilesToDelete.push_back(AliasedFile{});
-         aliasedFilesToDelete.back() = std::move(*iter);
-         iter = mAliasedFiles.erase(iter);
-      }
+   for( const auto &file : mAliasedFiles ) {
+      if (mFileListCtrl->GetItemState(i, wxLIST_STATE_SELECTED))
+         aliasedFilesToDelete.push_back( file );
       else
-         ++iter;
+         remainingAliasedFiles.push_back( file );
       ++i;
    }
 
+   // provides STRONG-GUARANTEE
    RemoveDependencies(mProject, aliasedFilesToDelete);
+
+   // COMMIT OPERATIONS needing NOFAIL-GUARANTEE:
+   mAliasedFiles.swap( remainingAliasedFiles );
    PopulateList();
 
    if (mAliasedFiles.empty() || !mHasNonMissingFiles)
