@@ -65,19 +65,7 @@ class ScreenFrame final : public wxFrame
    void OnToggleBackgroundWhite(wxCommandEvent & event);
 
    void DoCapture(wxString captureMode);
-   void OnCaptureWindowContents(wxCommandEvent & event);
-   void OnCaptureFullWindow(wxCommandEvent & event);
-   void OnCaptureWindowPlus(wxCommandEvent & event);
-   void OnCaptureFullScreen(wxCommandEvent & event);
-   void OnCaptureToolbars(wxCommandEvent & event);
-   void OnCaptureSelectionBar(wxCommandEvent & event);
-   void OnCaptureTools(wxCommandEvent & event);
-   void OnCaptureTransport(wxCommandEvent & event);
-   void OnCaptureMixer(wxCommandEvent & event);
-   void OnCaptureMeter(wxCommandEvent & event);
-   void OnCaptureEdit(wxCommandEvent & event);
-   void OnCaptureDevice(wxCommandEvent & event);
-   void OnCaptureTranscription(wxCommandEvent & event);
+   void OnCaptureSomething(wxCommandEvent & event);
 
    void TimeZoom(double seconds);
    void OnOneSec(wxCommandEvent & event);
@@ -91,12 +79,6 @@ class ScreenFrame final : public wxFrame
    void OnMedTracks(wxCommandEvent & event);
    void OnTallTracks(wxCommandEvent & event);
 
-   void OnCaptureTrackPanel(wxCommandEvent & event);
-   void OnCaptureRuler(wxCommandEvent & event);
-   void OnCaptureTracks(wxCommandEvent & event);
-   void OnCaptureFirstTrack(wxCommandEvent & event);
-   void OnCaptureSecondTrack(wxCommandEvent & event);
-
    std::unique_ptr<ScreenshotCommand> CreateCommand();
 
    wxCheckBox *mDelayCheckBox;
@@ -111,7 +93,11 @@ class ScreenFrame final : public wxFrame
    DECLARE_EVENT_TABLE()
 };
 
-using ScreenFramePtr = Destroy_ptr<ScreenFrame>;
+// Static pointer to the unique ScreenFrame window.
+// Formerly it was parentless, therefore this was a Destroy_ptr<ScreenFrame>
+// But now the window is owned, so just use a bare pointer, and null it when
+// the unique window is destroyed.
+using ScreenFramePtr = ScreenFrame*;
 ScreenFramePtr mFrame;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -119,7 +105,12 @@ ScreenFramePtr mFrame;
 void OpenScreenshotTools()
 {
    if (!mFrame) {
-      mFrame = ScreenFramePtr{ safenew ScreenFrame(NULL, -1) };
+      auto parent = wxGetApp().GetTopWindow();
+      if (!parent) {
+         wxASSERT(false);
+         return;
+      }
+      mFrame = ScreenFramePtr{ safenew ScreenFrame(parent, -1) };
    }
    mFrame->Show();
    mFrame->Raise();
@@ -127,7 +118,7 @@ void OpenScreenshotTools()
 
 void CloseScreenshotTools()
 {
-   mFrame.reset();
+   mFrame = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -177,34 +168,45 @@ enum
 
    IdDelayCheckBox,
 
-   IdCaptureToolbars,
+   IdCaptureFirst,
+   // No point delaying the capture of sets of things.
+   IdCaptureMenus= IdCaptureFirst,
+   IdCaptureEffects,
+   IdCapturePreferences,
+
+   // Put all events that need delay between AllDelayed and LastDelayed.
+   IdAllDelayedEvents,
+   IdCaptureToolbars  =IdAllDelayedEvents,
+   IdCaptureWindowContents,
+   IdCaptureFullWindow,
+   IdCaptureWindowPlus,
+   IdCaptureFullScreen,
+  
    IdCaptureSelectionBar,
+   IdCaptureSpectralSelection,
    IdCaptureTools,
    IdCaptureTransport,
    IdCaptureMixer,
    IdCaptureMeter,
+   IdCapturePlayMeter,
+   IdCaptureRecordMeter,
    IdCaptureEdit,
    IdCaptureDevice,
    IdCaptureTranscription,
+   IdCaptureScrub,
 
    IdCaptureTrackPanel,
    IdCaptureRuler,
    IdCaptureTracks,
    IdCaptureFirstTrack,
    IdCaptureSecondTrack,
+   IdCaptureLast = IdCaptureSecondTrack,
+
+   IdLastDelayedEvent,
 
    IdToggleBackgroundBlue,
    IdToggleBackgroundWhite,
 
-   // Put all events that might need delay below:
-   IdAllDelayedEvents,
-
-   IdCaptureWindowContents,
-   IdCaptureFullWindow,
-   IdCaptureWindowPlus,
-   IdCaptureFullScreen,
-
-   IdLastDelayedEvent,
 };
 
 BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
@@ -216,20 +218,7 @@ BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_BUTTON(IdMainWindowLarge,        ScreenFrame::OnMainWindowLarge)
    EVT_TOGGLEBUTTON(IdToggleBackgroundBlue,   ScreenFrame::OnToggleBackgroundBlue)
    EVT_TOGGLEBUTTON(IdToggleBackgroundWhite,  ScreenFrame::OnToggleBackgroundWhite)
-   EVT_BUTTON(IdCaptureWindowContents,  ScreenFrame::OnCaptureWindowContents)
-   EVT_BUTTON(IdCaptureFullWindow,      ScreenFrame::OnCaptureFullWindow)
-   EVT_BUTTON(IdCaptureWindowPlus,      ScreenFrame::OnCaptureWindowPlus)
-   EVT_BUTTON(IdCaptureFullScreen,      ScreenFrame::OnCaptureFullScreen)
-
-   EVT_BUTTON(IdCaptureToolbars,        ScreenFrame::OnCaptureToolbars)
-   EVT_BUTTON(IdCaptureSelectionBar,    ScreenFrame::OnCaptureSelectionBar)
-   EVT_BUTTON(IdCaptureTools,           ScreenFrame::OnCaptureTools)
-   EVT_BUTTON(IdCaptureTransport,       ScreenFrame::OnCaptureTransport)
-   EVT_BUTTON(IdCaptureMixer,           ScreenFrame::OnCaptureMixer)
-   EVT_BUTTON(IdCaptureMeter,           ScreenFrame::OnCaptureMeter)
-   EVT_BUTTON(IdCaptureEdit,            ScreenFrame::OnCaptureEdit)
-   EVT_BUTTON(IdCaptureDevice,          ScreenFrame::OnCaptureDevice)
-   EVT_BUTTON(IdCaptureTranscription,   ScreenFrame::OnCaptureTranscription)
+   EVT_COMMAND_RANGE(IdCaptureFirst, IdCaptureLast, wxEVT_COMMAND_BUTTON_CLICKED, ScreenFrame::OnCaptureSomething)
 
    EVT_BUTTON(IdOneSec,                 ScreenFrame::OnOneSec)
    EVT_BUTTON(IdTenSec,                 ScreenFrame::OnTenSec)
@@ -241,12 +230,6 @@ BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_BUTTON(IdMedTracks,              ScreenFrame::OnMedTracks)
    EVT_BUTTON(IdTallTracks,             ScreenFrame::OnTallTracks)
 
-   EVT_BUTTON(IdCaptureTrackPanel,      ScreenFrame::OnCaptureTrackPanel)
-   EVT_BUTTON(IdCaptureRuler,           ScreenFrame::OnCaptureRuler)
-   EVT_BUTTON(IdCaptureTracks,          ScreenFrame::OnCaptureTracks)
-   EVT_BUTTON(IdCaptureFirstTrack,      ScreenFrame::OnCaptureFirstTrack)
-   EVT_BUTTON(IdCaptureSecondTrack,     ScreenFrame::OnCaptureSecondTrack)
-
    EVT_BUTTON(IdDirChoose,              ScreenFrame::OnDirChoose)
 END_EVENT_TABLE();
 
@@ -294,11 +277,19 @@ ScreenFrame::ScreenFrame(wxWindow * parent, wxWindowID id)
    // Note that the audio could be playing.
    // The monitoring will switch off temporarily
    // because we've switched monitor mid play.
-   mContext.GetProject()->GetToolManager()->Reset();
+   // Bug 383 - Resetting the toolbars is not wanted.
+   // Any that are invisible will be amde visible as/when needed.
+   //mContext.GetProject()->GetToolManager()->Reset();
+   Center();
 }
 
 ScreenFrame::~ScreenFrame()
 {
+   if (this == mFrame)
+      mFrame = nullptr;
+   else
+      // There should only be one!
+      wxASSERT(false);
 }
 
 void ScreenFrame::Populate()
@@ -382,7 +373,18 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          S.StartHorizontalLay();
          {
             S.Id(IdCaptureToolbars).AddButton(_("All Toolbars"));
+#ifdef EXPERIMENTAL_DOCS_AUTOMATION
+            S.Id(IdCaptureMenus).AddButton(_("All Menus"));
+            S.Id(IdCaptureEffects).AddButton(_("All Effects"));
+            S.Id(IdCapturePreferences).AddButton(_("All Preferences"));
+#endif
+         }
+         S.EndHorizontalLay();
+
+         S.StartHorizontalLay();
+         {
             S.Id(IdCaptureSelectionBar).AddButton(_("SelectionBar"));
+            S.Id(IdCaptureSpectralSelection).AddButton(_("Spectral Selection"));
             S.Id(IdCaptureTools).AddButton(_("Tools"));
             S.Id(IdCaptureTransport).AddButton(_("Transport"));
          }
@@ -392,9 +394,17 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          {
             S.Id(IdCaptureMixer).AddButton(_("Mixer"));
             S.Id(IdCaptureMeter).AddButton(_("Meter"));
+            S.Id(IdCapturePlayMeter).AddButton(_("Play Meter"));
+            S.Id(IdCaptureRecordMeter).AddButton(_("Record Meter"));
+         }
+         S.EndHorizontalLay();
+
+         S.StartHorizontalLay();
+         {
             S.Id(IdCaptureEdit).AddButton(_("Edit"));
             S.Id(IdCaptureDevice).AddButton(_("Device"));
             S.Id(IdCaptureTranscription).AddButton(_("Transcription"));
+            S.Id(IdCaptureScrub).AddButton(_("Scrub"));
          }
          S.EndHorizontalLay();
 
@@ -463,16 +473,20 @@ bool ScreenFrame::ProcessEvent(wxEvent & e)
 {
    int id = e.GetId();
 
+   // If split into two parts to make for easier breakpoint
+   // when testing timer.
    if (mDelayCheckBox &&
        mDelayCheckBox->GetValue() &&
        e.IsCommandEvent() &&
-       e.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED &&
-       id >= IdAllDelayedEvents && id <= IdLastDelayedEvent &&
+       e.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED)
+   {
+      if( id >= IdAllDelayedEvents && id <= IdLastDelayedEvent &&
        e.GetEventObject() != NULL) {
-      // safenew because it's a one-shot that deletes itself
-      ScreenFrameTimer *timer = safenew ScreenFrameTimer(this, e);
-      timer->Start(5000, true);
-      return true;
+         // safenew because it's a one-shot that deletes itself
+         ScreenFrameTimer *timer = safenew ScreenFrameTimer(this, e);
+         timer->Start(5000, true);
+         return true;
+      }
    }
 
    if (e.IsCommandEvent() && e.GetEventObject() == NULL) {
@@ -483,8 +497,6 @@ bool ScreenFrame::ProcessEvent(wxEvent & e)
 
 void ScreenFrame::OnCloseWindow(wxCloseEvent &  WXUNUSED(event))
 {
-   if (this == mFrame.get())
-      mFrame.release();
    Destroy();
 }
 
@@ -554,7 +566,8 @@ void ScreenFrame::SizeMainWindow(int w, int h)
 
    mContext.GetProject()->Maximize(false);
    mContext.GetProject()->SetSize(16, 16 + top, w, h);
-   mContext.GetProject()->GetToolManager()->Reset();
+   //Bug383 - Toolbar Resets not wanted.
+   //mContext.GetProject()->GetToolManager()->Reset();
 }
 
 void ScreenFrame::OnMainWindowSmall(wxCommandEvent & WXUNUSED(event))
@@ -578,94 +591,40 @@ void ScreenFrame::DoCapture(wxString captureMode)
    Show();
 }
 
-void ScreenFrame::OnCaptureWindowContents(wxCommandEvent & WXUNUSED(event))
+void ScreenFrame::OnCaptureSomething(wxCommandEvent &  event)
 {
-   DoCapture(wxT("window"));
-}
+   int i = event.GetId() - IdCaptureFirst;
 
-void ScreenFrame::OnCaptureFullWindow(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("fullwindow"));
-}
+   wxArrayString Names;
 
-void ScreenFrame::OnCaptureWindowPlus(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("windowplus"));
-}
+   Names.Add(wxT("menus"));
+   Names.Add(wxT("effects"));
+   Names.Add(wxT("preferences"));
 
-void ScreenFrame::OnCaptureFullScreen(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("fullscreen"));
-}
+   Names.Add(wxT("toolbars"));
+   Names.Add(wxT("window"));
+   Names.Add(wxT("fullwindow"));
+   Names.Add(wxT("windowplus"));
+   Names.Add(wxT("fullscreen"));
+   Names.Add(wxT("selectionbar"));
+   Names.Add(wxT("spectralselection"));
+   Names.Add(wxT("tools"));
+   Names.Add(wxT("transport"));
+   Names.Add(wxT("mixer"));
+   Names.Add(wxT("meter"));
+   Names.Add(wxT("playmeter"));
+   Names.Add(wxT("recordmeter"));
+   Names.Add(wxT("edit"));
+   Names.Add(wxT("device"));
+   Names.Add(wxT("transcription"));
+   Names.Add(wxT("scrub"));
+   Names.Add(wxT("trackpanel"));
+   Names.Add(wxT("ruler"));
+   Names.Add(wxT("tracks"));
+   Names.Add(wxT("firsttrack"));
+   Names.Add(wxT("secondtrack"));
 
-void ScreenFrame::OnCaptureToolbars(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("toolbars"));
-}
-
-void ScreenFrame::OnCaptureSelectionBar(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("selectionbar"));
-}
-
-void ScreenFrame::OnCaptureTools(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("tools"));
-}
-
-void ScreenFrame::OnCaptureTransport(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("transport"));
-}
-
-void ScreenFrame::OnCaptureMixer(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("mixer"));
-}
-
-void ScreenFrame::OnCaptureMeter(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("meter"));
-}
-
-void ScreenFrame::OnCaptureEdit(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("edit"));
-}
-
-void ScreenFrame::OnCaptureDevice(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("device"));
-}
-
-void ScreenFrame::OnCaptureTranscription(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("transcription"));
-}
-
-void ScreenFrame::OnCaptureTrackPanel(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("trackpanel"));
-}
-
-void ScreenFrame::OnCaptureRuler(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("ruler"));
-}
-
-void ScreenFrame::OnCaptureTracks(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("tracks"));
-}
-
-void ScreenFrame::OnCaptureFirstTrack(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("firsttrack"));
-}
-
-void ScreenFrame::OnCaptureSecondTrack(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("secondtrack"));
+   DoCapture(Names[i]);
 }
 
 void ScreenFrame::TimeZoom(double seconds)
