@@ -6,8 +6,15 @@
 
   Leland Lucius
 
-**********************************************************************/
+********************************************************************//**
 
+\class AUControl
+\brief a wxControl with Cocoa/Carbon support
+
+\class AUControlImpl
+\brief a wxWidgetCocoaImpl 
+
+*//********************************************************************/
 #ifndef AUDACITY_AUCONTROL_H
 #define AUDACITY_AUCONTROL_H
 
@@ -33,6 +40,8 @@ public:
    AUControl();
    ~AUControl();
 
+   void Close();
+
    bool Create(wxWindow *parent, AudioComponent comp, AudioUnit unit, bool custom);
    void CreateCocoa();
    void CreateGeneric();
